
# q-expansion of newform 4304.2.a.c, downloaded from the LMFDB on 27 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4304
weight = 2
poly_data = [-3, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, -1], [1, -1], [1, 0], [-3, 0], [5, 0], [-1, -2], [-2, 0], [1, -4], [7, -1], [5, 2], [1, -2], [-4, -2], [3, 4], [3, 3], [0, 0], [5, -5], [-1, 0], [-7, -4], [6, -6], [1, -2], [5, -1], [-4, -2], [-5, -1], [0, 2], [0, 3], [5, -7], [7, -4], [5, 6], [-6, 3], [2, -1], [-16, -2], [15, -3], [11, 5], [10, -1], [11, 2], [6, 2], [11, -1], [-5, 11], [-11, 8], [-3, 12], [4, 4], [-10, 4], [9, -10], [21, -3], [10, -3], [-15, 7], [7, 3], [-12, 9], [-17, 10], [3, 6], [6, -3], [-5, 10], [-9, 9], [-3, 0], [-10, -5], [-1, 0], [-11, 0], [0, -7], [-3, 6], [13, 0], [6, 0], [3, -8], [0, -6], [-15, -1], [29, -2], [1, 0], [11, -15], [-21, 6], [-6, -1], [-14, 11], [0, 0], [16, 3], [30, -4], [16, -9], [10, 5], [-8, -4], [-9, 5], [23, 7], [-23, -2], [-21, -3], [3, 8], [19, 2], [17, 3], [-7, 11], [2, -8], [12, -3], [-10, 9], [-10, 10], [-29, -3], [14, -8], [-19, 4], [20, 8], [24, 6], [-3, -14], [6, 12], [-1, 13], [-21, 12], [21, 1], [-2, -8], [-10, -13], [20, -14], [-27, 12], [-27, 3], [-6, 16], [-13, -12], [-17, -1], [12, -12], [6, 6], [9, 11], [1, -15], [-27, -4], [-1, 13], [-10, -4], [8, -7], [1, 14], [10, -15], [-21, 12], [17, -14], [-1, 13], [5, 9], [11, 6], [17, 1], [-8, 2], [-8, 18], [-12, 3], [5, 12], [-24, 6], [-13, 8], [5, 0], [21, 1], [-12, 15], [-11, 15], [30, -16], [-30, 0], [1, -17], [-18, 18], [-33, 16], [30, -15], [-10, 16], [22, -6], [-6, 0], [3, -2], [6, 3], [9, -13], [15, -9], [-14, 1], [30, -18], [9, 1], [13, -19], [-10, 15], [4, -19], [-9, -14], [-15, -18], [-1, -4], [-6, 21], [-26, 6], [8, 10], [-32, 13], [0, -21], [-40, 7], [4, 8], [-16, -13], [24, -6], [-9, 15], [-5, -13], [-17, 15], [-35, 4], [-8, -2], [-25, 28], [-6, 30], [13, -29], [42, 6], [-31, 21], [-8, 3], [-49, 7], [-16, 2], [15, -6], [46, -3], [20, -21], [-19, 5], [44, 4], [38, -6], [-27, 18], [-27, 21], [-11, -10], [-8, -2], [-31, -10], [-11, 22], [5, 10], [27, -4], [-51, 12], [16, 18], [-18, -15], [31, -10], [-6, -15], [-23, 34], [-10, 9], [-33, 0], [-30, -6], [8, -14], [-30, 19], [27, -10], [30, -1], [-7, 4], [-28, -2], [-28, -7], [-18, 12], [-28, 19], [28, 12], [48, -7], [32, 7], [-34, 20], [24, -18], [16, -10], [16, -20], [14, 20], [-8, -4], [-32, 8], [-15, 12], [33, -16], [-43, -4], [-28, -5], [-9, -14], [-3, 15], [46, -14], [5, -23], [-23, 35], [5, -19], [-47, 2], [10, -26], [43, 9], [33, 4], [2, -26], [32, -10], [-11, 2], [17, 6], [25, -13], [-15, -18], [24, 12], [-63, 6], [-27, 4], [36, -8], [49, 4], [15, -18], [-50, 5], [39, -8], [-29, -1], [1, -24], [28, -25], [26, -12], [-15, 6], [43, 8], [16, -20], [-41, 20], [26, 7], [-11, 28], [37, 9], [6, -24], [-56, -8], [-5, -12], [20, -17], [-4, -27], [25, -17], [5, 7], [14, -19], [41, 10], [-3, 6], [-21, -5], [-27, 33], [-27, 35], [7, -9], [-13, 30], [-36, -17], [25, -32], [9, 13], [-34, -2], [-1, 9], [11, -18], [6, 9], [-39, 6], [58, 0], [-6, -6], [6, -31], [6, -14], [46, -25], [-10, 3], [54, -3], [3, 4], [-33, -12], [71, -8], [-54, 9], [-8, -1], [4, 11], [44, 0], [-53, -10], [-6, -5], [39, -18], [51, 3], [22, -21], [-32, 7], [-15, -3], [56, -10], [-38, 17], [20, 5], [-38, -17], [18, 6], [21, 14], [55, -4], [-2, 1], [10, 8], [24, -33], [26, -8], [-13, -16], [36, 18], [20, -9], [-62, 17], [37, 14], [-25, 42], [10, -10], [-9, 25], [47, -15], [-23, -10], [11, 26], [69, -3], [-23, 4], [71, 2], [-18, -11], [57, -6], [-2, 5], [-30, 41], [-28, 31], [-42, 13], [16, -16], [69, -11], [-39, 3], [35, -6], [45, 15], [67, -11], [39, -26], [56, -3], [1, -22], [54, -30], [-81, -2], [-48, 15], [4, 8], [-43, 36], [13, 24], [40, 14], [-22, -26], [94, 0], [0, -22], [-3, 27], [15, -17], [47, -24], [-46, 25], [9, -6], [-81, 0], [-17, -28], [-3, 0], [-56, 16], [41, -32], [51, -24], [8, -5], [34, -18], [53, 0], [71, 10], [52, 0], [-6, -25], [-18, 24], [-59, -15], [-55, 1], [-30, 24], [-66, -5], [32, -12], [-13, 22], [-9, 36], [-56, -8], [26, 16], [-5, 1], [27, 12], [-1, -14], [35, -16], [-36, -6], [-44, 12], [20, -23], [-13, 8], [-4, -9], [33, -2], [23, -14], [-1, -9], [33, 15], [30, -3], [2, 38], [10, -1], [-34, -24], [-33, 19], [16, -37], [53, -10], [45, -39], [77, 0], [-23, -28], [41, -16], [6, 0], [22, 8], [-39, 43], [-31, 30], [-34, 25], [8, 35], [-29, -16], [17, -45], [37, -31], [-41, -10], [-21, 10], [52, -23], [-15, 42], [35, 16], [7, -6], [30, -3], [23, -14], [-27, 30], [50, 15], [-12, 0], [-3, 6], [-51, 20], [-21, 27], [-14, 23], [-4, 1], [67, -24], [5, -20]]
